#include <M5StickC.h>
#include <WiFi.h>
#include "time.h"

#define TFT_GREY 0x5AEB

const char* ssid       = "**********"; // ご自分のWiFiルータのssidを書き込んでください
const char* password   = "**********"; // ご自分のWiFiルータのpasswordを書き込んでください
const char* ntpServer =  "ntp.jst.mfeed.ad.jp";
 
RTC_TimeTypeDef RTC_TimeStruct;
RTC_DateTypeDef RTC_DateStruct;

float sx = 0, sy = 1, mx = 1, my = 0, hx = -1, hy = 0;    // Saved H, M, S x & y multipliers
float sdeg=0, mdeg=0, hdeg=0;
uint16_t osx=120, osy=120, omx=120, omy=120, ohx=120, ohy=120;  // Saved H, M, S x & y coords
uint16_t x0=0, x1=0, yy0=0, yy1=0;

boolean initial = 1;

uint16_t yy,mt,dd,hh,mm,ss;

void setup(void) {
  M5.begin();
  M5.Lcd.setRotation(1);  
  M5.Lcd.fillScreen(TFT_GREY);
  M5.Lcd.setTextColor(TFT_BLACK, TFT_GREY);  // Adding a background colour erases previous text automatically
  M5.Lcd.setTextSize(1);
;
  // Draw clock face
  M5.Lcd.fillCircle(40, 40, 40, TFT_GREEN);
  M5.Lcd.fillCircle(40, 40, 36, TFT_BLACK);

  // Draw 12 lines
  for(int i = 0; i<360; i+= 30) {
    sx = cos((i-90)*0.0174532925);
    sy = sin((i-90)*0.0174532925);
    x0 = sx*38+40;
    yy0 = sy*38+40;
    x1 = sx*32+40;
    yy1 = sy*32+40;

    M5.Lcd.drawLine(x0, yy0, x1, yy1, TFT_GREEN);
  }

  // Draw 60 dots
  for(int i = 0; i<360; i+= 6) {
    sx = cos((i-90)*0.0174532925);
    sy = sin((i-90)*0.0174532925);
    x0 = sx*34+40;
    yy0 = sy*34+40;
    // Draw minute markers
    M5.Lcd.drawPixel(x0, yy0, TFT_WHITE);
    
    // Draw main quadrant dots
    if(i==0 || i==180) M5.Lcd.fillCircle(x0, yy0, 2, TFT_WHITE);
    if(i==90 || i==270) M5.Lcd.fillCircle(x0, yy0, 2, TFT_WHITE);
  }

  M5.Lcd.fillCircle(40, 40, 2, TFT_WHITE);

    // connect to WiFi
  Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println(" CONNECTED");
 
  // Set ntp time to local
  configTime(9 * 3600, 0, ntpServer);
 
  // Get local time
  struct tm timeInfo;
  if (getLocalTime(&timeInfo)) {

    // Set RTC time
    RTC_TimeTypeDef TimeStruct;
    TimeStruct.Hours   = timeInfo.tm_hour;
    TimeStruct.Minutes = timeInfo.tm_min;
    TimeStruct.Seconds = timeInfo.tm_sec;
    M5.Rtc.SetTime(&TimeStruct);
 
    RTC_DateTypeDef DateStruct;
    DateStruct.WeekDay = timeInfo.tm_wday;
    DateStruct.Month = timeInfo.tm_mon + 1;
    DateStruct.Date = timeInfo.tm_mday;
    DateStruct.Year = timeInfo.tm_year + 1900;
    M5.Rtc.SetData(&DateStruct);
  }

  //disconnect WiFi
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);
}

void loop() {
  static const char *wd[7] = {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
  // put your main code here, to run repeatedly:
  M5.Rtc.GetTime(&RTC_TimeStruct);
  M5.Rtc.GetData(&RTC_DateStruct);
  yy=RTC_DateStruct.Year; mt=RTC_DateStruct.Month; dd=RTC_DateStruct.Date;
  hh=RTC_TimeStruct.Hours; mm=RTC_TimeStruct.Minutes; ss=RTC_TimeStruct.Seconds;
  M5.Lcd.setCursor(90, 15); M5.Lcd.printf("%04d-%02d-%02d", yy, mt, dd);
  M5.Lcd.setCursor(90, 35); M5.Lcd.printf("%s", wd[RTC_DateStruct.WeekDay]);
  M5.Lcd.setCursor(90, 55); M5.Lcd.printf("%02d:%02d:%02d", hh, mm, ss);

  // Pre-compute hand degrees, x & y coords for a fast screen update
  sdeg = ss*6;                  // 0-59 -> 0-354
  mdeg = mm*6+sdeg*0.01666667;  // 0-59 -> 0-360 - includes seconds
  hdeg = hh*30+mdeg*0.0833333;  // 0-11 -> 0-360 - includes minutes and seconds
  hx = cos((hdeg-90)*0.0174532925);    
  hy = sin((hdeg-90)*0.0174532925);
  mx = cos((mdeg-90)*0.0174532925);    
  my = sin((mdeg-90)*0.0174532925);
  sx = cos((sdeg-90)*0.0174532925);    
  sy = sin((sdeg-90)*0.0174532925);

  if (ss==0 || initial) {
    initial = 0;
    // Erase hour and minute hand positions every minute
    M5.Lcd.drawLine(ohx, ohy, 40, 40, TFT_BLACK);
    ohx = hx*15+40;    
    ohy = hy*15+40;
    M5.Lcd.drawLine(omx, omy, 40, 40, TFT_BLACK);
    omx = mx*20+40;    
    omy = my*20+40;
  }

  // Redraw new hand positions, hour and minute hands not erased here to avoid flicker
  M5.Lcd.drawLine(osx, osy, 40, 40, TFT_BLACK);
  osx = sx*25+40;    
  osy = sy*25+40;
  M5.Lcd.drawLine(osx, osy, 40, 40, TFT_RED);
  M5.Lcd.drawLine(ohx, ohy, 40, 40, TFT_WHITE);
  M5.Lcd.drawLine(omx, omy, 40, 40, TFT_WHITE);
  M5.Lcd.drawLine(osx, osy, 40, 40, TFT_RED);

  M5.Lcd.fillCircle(40, 40, 2, TFT_RED);

    
  delay(500);
}
